/*!
 * @file        main.c
 *
 * @brief       Main program body
 *
 * @version     V1.0.0
 *
 * @date        2023-07-31
 *
 * @attention
 *
 *  Copyright (C) 2023 Geehy Semiconductor
 *
 *  You may not use this file except in compliance with the
 *  GEEHY COPYRIGHT NOTICE (GEEHY SOFTWARE PACKAGE LICENSE).
 *
 *  The program is only for reference, which is distributed in the hope
 *  that it will be useful and instructional for customers to develop
 *  their software. Unless required by applicable law or agreed to in
 *  writing, the program is distributed on an "AS IS" BASIS, WITHOUT
 *  ANY WARRANTY OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the GEEHY SOFTWARE PACKAGE LICENSE for the governing permissions
 *  and limitations under the License.
 */

/* Includes */
#include "main.h"
#include "Board.h"
#include "apm32f4xx_tmr.h"
#include "apm32f4xx_comp.h"
#include "apm32f4xx_gpio.h"

/** @addtogroup Examples
  @{
  */

/** @addtogroup COMP_PWMSignalControl
  @{
  */

/** @defgroup COMP_PWMSignalControl_Functions Functions
  @{
  */

/* Delay */
void Delay(uint32_t count);
/* COMP init */
void COMP_Init(void);
/* TMR init */
void TMR_Init(void);

/*!
 * @brief       Main program
 *
 * @param       None
 *
 * @retval      None
 */
int main(void)
{
    TMR_Init();

    COMP_Init();

    while (1)
    {
    }
}

/*!
 * @brief       Delay
 *
 * @param       count:  delay count
 *
 * @retval      None
 *
 * @note
 */
void Delay(uint32_t count)
{
    volatile uint32_t delay = count;

    while (delay--);
}

/*!
 * @brief       TMR Init
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void TMR_Init()
{
    GPIO_Config_T      GPIO_ConfigStruct;
    TMR_BDTConfig_T    BDTConfig;
    TMR_BaseConfig_T   TMR_TimeBaseStruct;
    TMR_OCConfig_T     OCcongigStruct;
    
    /* Enable GPIOA clock */
    RCM_EnableAHB1PeriphClock(RCM_AHB1_PERIPH_GPIOA);
    
    /* Enable TMR1 clock */
    RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_TMR1);
    
    /* TMR1 channel 1 (PA8) configuration */
    GPIO_ConfigStructInit(&GPIO_ConfigStruct);
    GPIO_ConfigStruct.mode  = GPIO_MODE_AF;
    GPIO_ConfigStruct.otype = GPIO_OTYPE_PP;
    GPIO_ConfigStruct.pin   = GPIO_PIN_8;
    GPIO_ConfigStruct.speed = GPIO_SPEED_50MHz;
    GPIO_Config(GPIOA,&GPIO_ConfigStruct);
    
    /* Enable PA8 alternate function for TMR1 */
    GPIO_ConfigPinAF(GPIOA,GPIO_PIN_SOURCE_8,GPIO_AF_TMR1);
    
    /* TMR base configuration */
    TMR_ConfigTimeBaseStructInit(&TMR_TimeBaseStruct);
    TMR_TimeBaseStruct.clockDivision     = TMR_CLOCK_DIV_1;
    TMR_TimeBaseStruct.countMode         = TMR_COUNTER_MODE_UP;
    TMR_TimeBaseStruct.division          = 0;
    TMR_TimeBaseStruct.period            = 999;
    TMR_TimeBaseStruct.repetitionCounter = 0;
    TMR_ConfigTimeBase(TMR1,&TMR_TimeBaseStruct);
    
    /* PWM mode is channel 1 Configuration */
    TMR_ConfigOCStructInit(&OCcongigStruct);
    OCcongigStruct.idleState    = TMR_OC_IDLE_STATE_RESET;
    OCcongigStruct.mode         = TMR_OC_MODE_PWM1;
    OCcongigStruct.nIdleState   = TMR_OC_NIDLE_STATE_RESET;
    OCcongigStruct.nPolarity    = TMR_OC_NPOLARITY_HIGH;
    OCcongigStruct.outputNState = TMR_OC_NSTATE_ENABLE;
    OCcongigStruct.outputState  = TMR_OC_STATE_ENABLE;
    OCcongigStruct.polarity     = TMR_OC_POLARITY_HIGH;
    OCcongigStruct.pulse        = 500;
    TMR_ConfigOC1(TMR1,&OCcongigStruct);
    
    /* Automatic Output enable, Break, dead time and lock configuration*/
    TMR_ConfigBDTStructInit(&BDTConfig);
    BDTConfig.automaticOutput = TMR_AUTOMATIC_OUTPUT_ENABLE;
    BDTConfig.BRKPolarity     = TMR_BRK_POLARITY_HIGH;
    BDTConfig.BRKState        = TMR_BRK_STATE_ENABLE;
    BDTConfig.deadTime        = 0x01;
    BDTConfig.IMOS            = TMR_IMOS_STATE_ENABLE;
    BDTConfig.lockLevel       = TMR_LOCK_LEVEL_OFF;
    BDTConfig.RMOS            = TMR_RMOS_STATE_ENABLE;
    TMR_ConfigBDT(TMR1,&BDTConfig);
    
    /* Enable PWM output */
    TMR_EnablePWMOutputs(TMR1);
    
    /* Enable TMR1 */
    TMR_Enable(TMR1);
}

void COMP_Init(void)
{
    GPIO_Config_T      gpioConfig;
    COMP_Config_T      compConfig;
    
    /* Enable GPIOC clock */
    RCM_EnableAHB1PeriphClock(RCM_AHB1_PERIPH_GPIOC);
    
    /* Configure PC2 is connected to COMP2 non inverting input */
    GPIO_ConfigStructInit(&gpioConfig);
    gpioConfig.mode  = GPIO_MODE_AN;
    gpioConfig.pupd  = GPIO_PUPD_NOPULL;
    gpioConfig.pin   = GPIO_PIN_2;
    GPIO_Config(GPIOC,&gpioConfig);
    
    /* Enable COMP clock */
    RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_SYSCFG);
    
    /*  Configure COMP2 */
    COMP_ConfigStructInit(&compConfig);
    compConfig.invertingInput = COMP_INVERTING_INPUT_VREFINT;
    compConfig.mode           = COMP_MODE_LOWSPEED;
    compConfig.output         = COMP_OUTPUT_TIM1BKIN;
    compConfig.outputPol      = COMP_OUTPUTPOL_NONINVERTED;
    COMP_Config(COMP_SELECT_COMP2,&compConfig);
    
    /* Enable COMP2 */
    COMP_Enable(COMP_SELECT_COMP2);
}


/**@} end of group COMP_PWMSignalControl_Functions */
/**@} end of group COMP_PWMSignalControl */
/**@} end of group Examples */
